"use strict";
function set_deep(parent, keys, value) {
    for (var i = 0; i < keys.length - 1; i++)
        parent = parent[keys[i]] || (parent[keys[i]] = {});
    parent[keys[i]] = value;
}
/**
 * Combine classes, interfaces, and modules
 *
 * @param file
 * @returns {Symbols}
 */
function members(file) {
    var acc = {
        classes: {},
        modules: {}
    };
    Object.keys(file).forEach(function (name) {
        var symbol = file[name];
        // Class and constructor
        if (symbol.jsdoc.tags.some(function (t) { return t.title === 'interface'; }) ||
            symbol.jsdoc.tags.some(function (t) { return t.title === 'constructor'; })) {
            acc.classes[name] = { '': symbol };
        }
        else if (name.indexOf('.prototype.') !== -1) {
            var parts = name.split('.prototype.');
            var className = parts[0];
            var memberName = parts[1];
            if (acc.classes[className])
                acc.classes[className][memberName] = symbol;
        }
        else {
            var dot = name.lastIndexOf('.');
            var moduleName = name.substring(0, dot);
            var memberName = name.substring(dot + 1);
            if (!acc.modules[moduleName])
                acc.modules[moduleName] = {};
            acc.modules[moduleName][memberName] = symbol;
        }
    });
    return acc;
}
exports.members = members;
