/// <reference path="../index/node.d.ts"/>
"use strict";
var fs = require('fs');
var parser = require('./parser');
var combine = require('./combine');
var options = require('./options');
var fileByProvide = {};
var providesByFile = {};
function parent_symbol(name) {
    var dot = name.lastIndexOf('.');
    return name.substring(0, dot);
}
/**
 * @param symbolName
 * @returns file that goog.provide's name
 */
function file(symbolName) {
    if (symbolName)
        return fileByProvide[symbolName] || file(parent_symbol(symbolName));
    else
        return null;
}
exports.file = file;
/**
 * @param fileName
 * @returns value of name
 */
function symbols(fileName) {
    if (!providesByFile[fileName])
        providesByFile[fileName] = combine.members(parser.jsdoc(fs.readFileSync(fileName, 'utf8')));
    if (providesByFile[fileName])
        return providesByFile[fileName];
}
exports.symbols = symbols;
function symbol(symbolName) {
    var fileName = file(symbolName);
    if (fileName) {
        var s = symbols(fileName);
        return s.classes[symbolName] || s.modules[symbolName];
    }
    else
        return null;
}
exports.symbol = symbol;
function member(symbolName) {
    var fileName = file(symbolName);
    if (fileName) {
        var s = symbols(fileName);
        var dot = symbolName.lastIndexOf('.');
        var moduleName = symbolName.substring(0, dot);
        var memberName = symbolName.substring(dot + 1);
        var moduleValue = s.modules[moduleName] || {};
        return moduleValue[memberName] || null;
    }
    else
        return null;
}
exports.member = member;
// If user gives a TSV file with provided symbols, use that
if (options.provides) {
    fs.readFileSync(options.provides, 'utf8').split('\n').forEach(function (line) {
        var columns = line.split('\t');
        var file = columns[0];
        var symbol = columns[1];
        fileByProvide[symbol] = file;
    });
}
else {
    options.todo.forEach(function (todo) {
        var found = symbols(todo.input);
        var classNames = Object.keys(found.classes);
        var moduleNames = Object.keys(found.modules);
        var names = classNames.concat(moduleNames);
        names.forEach(function (name) { return fileByProvide[name] = todo.input; });
    });
}
